"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskExecutor = void 0;
const math_1 = require("@do-while-for-each/math");
const common_1 = require("./common");
class TaskExecutor {
    constructor(env) {
        this.env = env;
    }
    run(tasks) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const task of tasks)
                yield this.runTask(task);
        });
    }
    runTask(task) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            this.startTask(task);
            const script = [
                ...(task.beforeScript || []),
                ...task.getScript(),
                ...(task.afterScript || []),
            ];
            for (const command of script) {
                this.debug(command);
                const page = task.page;
                try {
                    switch (command.type) {
                        case 'newPage': {
                            task.page = yield this.env.newPage();
                            (_a = task.initReqInterceptors) === null || _a === void 0 ? void 0 : _a.call(task);
                            break;
                        }
                        case 'closePage': {
                            (_b = task.page) === null || _b === void 0 ? void 0 : _b.close();
                            delete task.page;
                            break;
                        }
                        case 'goto': {
                            yield page.goto(this.getUrl(command.data));
                            break;
                        }
                        case 'screenshot': {
                            yield ((_c = task.actionsBeforeScreenshot) === null || _c === void 0 ? void 0 : _c.call(task));
                            const buf = yield page.screenshot();
                            task.setScreenshot(buf);
                            if (command.data.save)
                                this.storage.set(task, { type: 'screenshot' }, buf);
                            break;
                        }
                        case 'compareScreenshot': {
                            if (this.options.screenshot.type !== 'png')
                                throw new Error(`Поддерживается только 'png' при сравнении скриншотов`);
                            const origImgBuf = this.storage.get(task, { type: 'screenshot' }).buf;
                            const screenshot = yield task.screenshot();
                            const compareResult = this.pngUtils.compareBuf(origImgBuf, screenshot);
                            task.setCompareScreenshotResult(compareResult);
                            break;
                        }
                        case 'wait': {
                            if (math_1.isNumber(command.data))
                                yield page.waitForTimeout(command.data);
                            break;
                        }
                        case 'waitForAllDataReceived': {
                            yield task.allDataReceived();
                            break;
                        }
                        case 'click': {
                            const { selector, options, useFullSelector } = command.data;
                            yield page.click(this.getSelector(selector, useFullSelector), options);
                            break;
                        }
                        case 'fill': {
                            const { selector, value, options, useFullSelector } = command.data;
                            yield page.fill(this.getSelector(selector, useFullSelector), value, options);
                            break;
                        }
                        case 'mouseClick': {
                            const { point, options } = command.data;
                            yield page.mouse.click(point[0], point[1], options);
                            break;
                        }
                        case 'login': {
                            yield task.login();
                            break;
                        }
                        default:
                            throw new Error(`Для команды '${command['type']}' не определен обработчик`);
                    }
                }
                catch (e) {
                    this.finishTask(task);
                    console.log(`---`);
                    console.log(`Ошибка при выполнении задачи '${task.id}'`);
                    console.log(script);
                    throw e;
                }
            }
            this.finishTask(task);
        });
    }
    /**
     * Для упрощения поддержки по умолчанию считать, что на целевом элементе
     * определен атрибут 'data-qa' со значением selector, например:
     *  <input data-qa="login-page_username" />
     */
    getSelector(selector, useFullSelector) {
        return useFullSelector ? selector : `[data-qa=${selector}]`;
    }
    getUrl(path) {
        return common_1.joinUrl(path, this.options.baseUrl);
    }
    //region Support
    get options() {
        return this.env.options;
    }
    get storage() {
        return this.env.storage;
    }
    get pngUtils() {
        return this.env.pngUtils;
    }
    debug(...args) {
        return this.env.debug(...args);
    }
    startTask(task) {
        task.isActive = true;
        this.debug(`==================================================`);
        this.debug(`start ${task.id} for '${this.env.id}'`);
        if (this.options.isDebug)
            console.time(task.id);
    }
    finishTask(task) {
        task.isActive = false;
        task.isFinished = true;
        if (this.options.isDebug)
            console.timeEnd(task.id);
    }
}
exports.TaskExecutor = TaskExecutor;
