"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileProcess = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
class FileProcess {
    static run(tasks) {
        tasks.forEach(([cmd, [src, dest]]) => {
            switch (cmd) {
                case 'clean-dir':
                    FileProcess.cleanDir(src, { recursive: true });
                    console.log(`> clean dir '${src}' \r\n`);
                    return;
                case 'move-file':
                    FileProcess.moveFile(src, dest);
                    console.log(`> move file '${src}' -> '${dest}' \r\n`);
                    return;
                case 'copy-file':
                    FileProcess.copyFile(src, dest);
                    console.log(`> copy file '${src}' -> '${dest}' \r\n`);
                    return;
                case 'delete-path':
                    FileProcess.deletePath(src, { recursive: true });
                    console.log(`> delete path '${src}' \r\n`);
                    return;
                default:
                    throw new Error(`unknown command type '${cmd}'`);
            }
        });
    }
    static cleanDir(source, options) {
        const fileNames = fs_1.readdirSync(source);
        fileNames.forEach(file => {
            const path = path_1.join(source.toString(), file);
            FileProcess.deletePath(path, options);
        });
    }
    static copyFile(src, dest, isMove = false, flags) {
        try {
            fs_1.copyFileSync(src, dest);
            if (isMove)
                fs_1.unlinkSync(src);
        }
        catch (e) {
            fs_1.closeSync(fs_1.openSync(dest, 'w'));
            console.error(e);
        }
    }
    static moveFile(src, dest, flags) {
        FileProcess.copyFile(src, dest, true, flags);
    }
    static deletePath(path, options) {
        if (fs_1.lstatSync(path).isDirectory())
            fs_1.rmdirSync(path, options);
        else
            fs_1.unlinkSync(path);
    }
}
exports.FileProcess = FileProcess;
