/// <reference types="node" />
import { IFileMetadata, IStorage, IStorageGet, ITask } from './contract';
import { AutomationEnvironment } from './automation.environment';
/**
 * Хранилище реализует структуру вида:
 *
 *  [environment]
 *     |---[screenshot]
 *            |--- task-1.png
 *            |--- task-2.png
 *            |--- ...
 *     |---[response]
 *            |--- [task-1]
 *                    |--- index.json  - индексный файл содержимого директории
 *                    |--- file1
 *                    |--- file2
 *                    |--- ...
 *                    |--- fileN
 *            |--- [task-2]
 *                    |--- index.json
 *                    |--- file1
 *                    |--- file2
 *                    |--- ...
 *                    |--- fileN
 *            |--- ...
 */
export declare class Storage implements IStorage {
    private env;
    constructor(env: AutomationEnvironment);
    get(task: ITask, meta: IFileMetadata): IStorageGet;
    set(task: ITask, meta: IFileMetadata, buf: Buffer): void;
    private initDirs;
    /**
     * Перед тем, как выполнить операцию в хранилище, надо спозиционироваться в нужное место:
     *  - убедиться, что нужные папки созданы;
     *  - обновить соответствующие переменные.
     */
    private updatePosition;
    private get dir();
    private get environmentDir();
    private storageFileTypeDir;
    private getStorageFileTypeDir;
    private taskDir;
    private taskIndexFilePath;
    private indexFileName;
    private getTaskDir;
    private file;
    private get currentIndex();
    private getIndex;
    private updateIndex;
    clean(): void;
    private cleanDir;
    private removeFiles;
    private get options();
    private debug;
}
//# sourceMappingURL=storage.d.ts.map